----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    01:52:36 08/09/2014 
-- Design Name: 
-- Module Name:    sistema_completo - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity sistema_completo is
  port ( 
	in_plaintext : in  std_logic_vector (0 to 63);
   habilita : in  std_logic;
   rst : in  std_logic;
   clk : in  std_logic;
   decipher : in  std_logic;
   rdy_outkey : out  std_logic;
   outkey : out  std_logic_vector(0 to 63);  --Clave de salida obtenida
   inkey : in  std_logic_vector ( 0 to 63);	
	pruebas_aleatorias : in  std_logic;
	error	: in  std_logic_vector (0 to 31);
	procesos_total : out  std_logic_vector (0 to 9);
	n_ataques_total : out  std_logic_vector (0 to 14)
   );
	
end sistema_completo;

architecture Behavioral of sistema_completo is

  component generator_plaintext
    port ( 
		in_plaintext : in  std_logic_vector (0 to 63);
      clk : in  std_logic;
      rst : in  std_logic;
      ready : in  std_logic;
		ready2 : in  std_logic;
      habilita : in  std_logic;
      rdy	: out std_logic;
      numero_ataques : out  std_logic_vector (0 to 3);
      out_plaintext : out  std_logic_vector (0 to 63)
		);

  end component;

  component top_key
    port(     
      outdata_gold : in  std_logic_vector (0 to 63);    --salida encriptada correcta (seal final)
      outdata_faulty : in  std_logic_vector (0 to 63);   -- salida encriptada fallida  (seal final ), aunque podemos 
      outkey : out std_logic_vector (0 to 63);
      habilita : in std_logic;
      clk : in  std_logic;
      rst : in  std_logic;
      rdy_subkey_round16 : out  std_logic;
      rdy :out  std_logic;
		pruebas_aleatorias : in  std_logic
      );
  end component;
  
  component des56
    port( 
      indata : in  std_logic_vector (0 to 63);
      inkey : in  std_logic_vector (0 to 63);
      outdata_gold : out  std_logic_vector (0 to 63);
      outdata_faulty : out  std_logic_vector (0 to 63);
		error : in  std_logic_vector (0 to 31);
		pruebas_aleatorias : in  std_logic;
      decipher : in  std_logic;
      numero_ataques	: in  std_logic_vector (0 to 3);
      clk : in  std_logic;
      rst : in  std_logic;
      rdy_next_next_cycle : out  std_logic;	-- output will be ready in two clock cycles - optional signal
      rdy_next_cycle : out  std_logic;      -- output will be ready in one clock cycle - optional signal
      rdy : out  std_logic;       -- output is ready NOW
      ready : in  std_logic
      );  
  end component;
  
  
  component generator_bits_inkey 
    generic(Valsat:integer:=255 );
    port ( 
		inkey : in  std_logic_vector (0 to 63);
      ready_subkey_round16 : in  std_logic;
      ready_comp : in  std_logic;
      rdy_key_generated : out std_logic;
      clk  : in  std_logic;
      rst  : in  std_logic;
      outkey_generated : out  std_logic_vector (0 to 63)
		);
  end component;
  
  component des56_basico 
    port( 
      indata : in  std_logic_vector (0 to 63);
      inkey : in  std_logic_vector (0 to 63);
      outdata : out  std_logic_vector (0 to 63);
      decipher : in  std_logic;
      ds : in  std_logic;
      clk : in  std_logic;
      rst : in  std_logic;
      rdy_next_next_cycle : out  std_logic;	-- output will be ready in two clock cycles - optional signal
      rdy_next_cycle : out  std_logic;      -- output will be ready in one clock cycle - optional signal
      rdy : out  std_logic       -- output is ready NOW
      );
  end component;

  component compara_salidas
    port ( 
		data_gold : in  std_logic_vector (0 to 63);
      data_prueba : in  std_logic_vector (0 to 63);
      clk : in  std_logic;
      rst : in  std_logic;
      rdy_outkey : out  std_logic;
      ready : in  std_logic;
      rdy : out  std_logic
		);
  end component;
  

  signal rdy_aux1 , rdy_aux2 , rdy_aux3 : std_logic;
  signal rdy_round16_aux : std_logic ;
  signal rdy_comp_aux : std_logic;
  signal rdy_key_generated_aux : std_logic;
  signal rdy_outkey_aux : std_logic;
  signal ready2_aux, p_ready2_aux : std_logic;
  
  signal plaintext_aux : std_logic_vector (0 to 63);

  signal outdata_gold_aux , outdata_faulty_aux : std_logic_vector (0 to 63);
  signal outkey_aux : std_logic_vector (0 to 63);
  signal outkey_generated_aux : std_logic_vector (0 to 63);
  signal outdata_aux : std_logic_vector (0 to 63) ;
  
 
  signal procesos, p_procesos : std_logic_vector (0 to 9);
  signal ataques_total, p_ataques_total : std_logic_vector (0 to 14);
  signal n_ataques_aux : std_logic_vector (0 to 3);
  signal ataques, p_ataques : std_logic_vector (0 to 3);
  
begin
  
  My_generator_plaintext: generator_plaintext
    port map(
		clk => clk, 
		rst => rst, 
		in_plaintext => in_plaintext, 
		ready => rdy_aux3, 
		habilita => habilita, 
		rdy => rdy_aux1, 
		ready2 => ready2_aux, 
		out_plaintext => plaintext_aux, 
		numero_ataques => n_ataques_aux
		);

  My_des56 : des56
    port map (
		clk => clk, 
		rst => rst, 
		indata => plaintext_aux, 
		inkey => inkey, 
		outdata_gold =>outdata_gold_aux, 
		outdata_faulty =>outdata_faulty_aux,
		error => error, 
		pruebas_aleatorias => pruebas_aleatorias, 
		decipher => decipher, 
		rdy => rdy_aux2, 
		ready => rdy_aux1, 
		numero_ataques => n_ataques_aux
		);

  My_top_key : top_key 
    port map (
		clk => clk, 
		rst => rst, 
		outdata_gold => outdata_gold_aux, 
		outdata_faulty => outdata_faulty_aux, 
		outkey => outkey_aux, 
		habilita => rdy_aux2, 
		rdy => rdy_aux3, 
		rdy_subkey_round16 => rdy_round16_aux, 
		pruebas_aleatorias => pruebas_aleatorias
		);
		
  My_generator_bits_inkey : generator_bits_inkey
    generic map(Valsat => 255 )
    port map (
		clk => clk, 
		rst => rst, 
		inkey => outkey_aux, 
		ready_subkey_round16 => rdy_round16_aux, 
		ready_comp => rdy_comp_aux, 
		rdy_key_generated => rdy_key_generated_aux, 
		outkey_generated => outkey_generated_aux
		);
  
  My_des56_basico : des56_basico
    port map (
		clk => clk, 
		rst => rst, 
		indata => plaintext_aux, 
		inkey => outkey_generated_aux, 
		outdata => outdata_aux, 
		decipher => decipher, 
		rdy => rdy_outkey_aux, 
		ds => rdy_key_generated_aux
		);

  My_compara_salidas : compara_salidas
    port map (
		clk => clk, 
		rst => rst, 
		data_gold => outdata_gold_aux, 
		data_prueba => outdata_aux, 
		ready => rdy_outkey_aux, 
		rdy => rdy_comp_aux, 
		rdy_outkey => rdy_outkey
		);
  
  outkey <= outkey_generated_aux;
  
  numero_ataques : process(procesos, rdy_round16_aux, n_ataques_aux)
  begin
    
    p_ataques_total <= ataques_total;
    
    if (procesos < 1000) then
      if (rdy_round16_aux = '1') then
        p_ready2_aux <= '1';			
        p_ataques <= n_ataques_aux;
        p_procesos <= procesos + 1;
        p_ataques_total <= ataques_total + n_ataques_aux;    
      else
        p_ataques <= ataques;
        p_procesos <= procesos;
        p_ready2_aux <= '0';     
      end if;
    else
      p_ready2_aux <= '0';
    end if;
    
  end process;
  
  sinc:process(rst ,clk)
  begin
    if(rst ='1')then
      ataques <= (others => '0');
      procesos <= (others => '0');
      ready2_aux <= '0';
      ataques_total <= (others => '0');  
    elsif(clk = '1' and clk'event)then
      ataques <= p_ataques;
      procesos <= p_procesos;
      ready2_aux <= p_ready2_aux;
      ataques_total <= p_ataques_total;    
    end if;
    
  end process;
  
  procesos_total <= procesos;
  n_ataques_total <= ataques_total;

 
end Behavioral;

