----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    21:02:35 04/28/2014 
-- Design Name: 
-- Module Name:    top_key - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity top_key IS

  generic(Valsat:integer:=63 );

  port( 
    outdata_gold : in  std_logic_vector (0 to 63);    --salida encriptada correcta 
    outdata_faulty : in  std_logic_vector (0 to 63);   -- salida encriptada fallida  
    outkey : out  std_logic_vector (0 to 63);
    habilita :  in  std_logic;
    clk : in  std_logic;
    rst : in  std_logic;
    rdy_subkey_round16 : out  std_logic;
    rdy : out  std_logic;
	 pruebas_aleatorias : in  std_logic
    );

end top_key ;

architecture Behavioral of top_key is

  subtype subkey is std_logic_vector(0 to 5);  --Subclave de 6 bits/caja de sustitucion
  
  type memory_mykey is array(integer range 0 to 7, integer range 0 to 20) of subkey;  
  signal mem , p_mem: memory_mykey;
  signal mem1 , p_mem1 , mem2 ,p_mem2  , mem_coincide, p_mem_coincide: memory_mykey;

-- Procedimiento que copia la columna "col" de "orig" en "dest"
  procedure copycolumn (constant orig: in memory_mykey;     
                        constant col: in integer;
                        signal dest : out memory_mykey) is
  begin
    for i in 0 to 20 loop					
      dest(col,i) <= orig(col,i);
    end loop;
  end procedure;
  
 -- Procedimiento que borra una tabla asignandole un valor no asignado en todas las posiciones
  procedure remove_array (signal table_key: inout memory_mykey) is  

  begin
    for i in 0 to 7 loop
      for j in 0 to 20 loop
        table_key(i,j) <= "UUUUUU";
      end loop;
    end loop;
  end procedure;

  component key_generator
    generic(Valsat:integer:=63 );
    port( 
      outkey : out std_logic_vector (0 to 5);
      fin_key : out  std_logic;
      habilita : in std_logic;
      clk  : in  std_logic;
      rst : in  std_logic  
      );
  end component;

  component s_box
    port( 
      subkey : in  std_logic_vector (0 to 5); 
      inmsg : in  std_logic_vector (0 TO 31);
      s_out : out  std_logic_vector ( 0 to 31);
      clk : in  std_logic;
      rst : in  std_logic 
      );
  end component;

  type estado is (inicio , captura_datos,  inicia_key , comprueba_key , 
						ciclo_espera , almacena_key, compara_key , calcula_key_entrada , fin);
  signal estado_actual , estado_siguiente : estado;

  type subestado is (calcula_outmsg , calcula_diferencia , calcula_dif_out_Feistel , calcula_p_inversa);
  signal operacion , p_operacion : subestado;

  signal p_outkey : std_logic_vector (0 to 63);
  signal diferencia , p_diferencia : std_logic_vector (0 to 63); --diferencia xor entre
  signal error , p_error, dif_out_Feistel , p_dif_out_Feistel : std_logic_vector (0 to 31 );
  signal outmsg_gold , p_outmsg_gold , outmsg_faulty , p_outmsg_faulty: std_logic_vector (0 to 63);
  signal p_inversa , p_p_inversa : std_logic_vector ( 0 to 31 );  --permutacion inversa = salidas de las cajas de sustitucion
  signal habilita_cont : std_logic ;	
  signal p_habilita_cont , p_p_habilita_cont : std_logic ;
  signal key_aux : std_logic_vector (0 to 5);
  signal fin_key_aux : std_logic;
  signal key1_gold , p_key1_gold: std_logic_vector (0 to 5);
  signal s_aux1 , s_aux2 : std_logic_vector (0 to 31 );
  signal outkey16 , p_outkey16 : std_logic_vector (0 to 47);	--Clave de la ronda 16

  
------Tabla compuesta de indices para recorrer las tablas de subclaves de cada caja de sustitucion------------
  subtype indice is integer range 0 to 21;  
  type tabla_indice is array(integer range 0 to 7) of indice;  
  signal i, j, k, p_i, p_j, p_k , indice_aux , p_indice_aux: tabla_indice;

-------Tabla usada como indice para identificar los errores de cada caja de sustitucion
  subtype indice2 is integer range 0 to 31;
  type tabla_indice2 is array(integer range 0 to 33) of indice2;
  signal indice_error :tabla_indice2;  

  subtype rdy_subkey0 is std_logic; 
  type tabla_rdy is array(integer range 0 to 7) of rdy_subkey0;  
  signal rdy_subkey, p_rdy_subkey: tabla_rdy;

  signal primera_inyeccion , p_primera_inyeccion: std_logic;
  signal p_rdy : std_logic;
  signal mykey: std_logic_vector (0 to 47);
  signal keylr : std_logic_vector (0 to 55);
  signal key_l , key_r : std_logic_vector (0 to 27);

  signal p_rdy_subkey_round16 :std_logic;

begin
  My_generator: key_generator
    generic map(Valsat => 63 )
    port map(
		clk => clk, 
		rst => rst, 
		habilita => habilita_cont, 
		outkey => key_aux, 
		fin_key => fin_key_aux
		);

  My_s_box1: s_box
    port map(
		clk => clk, 
		rst => rst, 
		subkey => key_aux, 
		s_out => s_aux1, 
		inmsg => outmsg_gold (32 to 63)
		);

  My_s_box2: s_box
    port map(
		clk => clk, 
		rst => rst, 
		subkey => key_aux, 
		s_out => s_aux2, 
		inmsg => outmsg_faulty (32 to 63)
		);

  stop_key : process (habilita_cont ,estado_actual , operacion ,  error , s_aux1 , s_aux2, p_inversa, key_aux, 
							 outdata_gold , outdata_faulty ,outmsg_gold , outmsg_faulty , diferencia , dif_out_Feistel ,
                      i, j, k, indice_error , indice_aux, rdy_subkey, mem , mem1 , mem2,mem_coincide, 
                      outkey16, mykey, keylr , key_l, key_r) 
  begin
    
    p_indice_aux <= indice_aux; 
    p_primera_inyeccion <= primera_inyeccion; 
    p_i <= i;
    p_j <= j;
    p_k <= k;
    p_rdy_subkey <= rdy_subkey;
    p_habilita_cont <= habilita_cont;
    p_operacion <= operacion;
    p_mem <= mem;
    p_mem1 <= mem1;
    p_mem2 <= mem2;
    p_mem_coincide <= mem_coincide;
    
    case estado_actual is      
      
      when inicio =>						
        p_rdy <= '0';
		  p_rdy_subkey_round16 <= '0';

        for c in 0 to 7 loop
          p_rdy_subkey(c) <= '0';		
        end loop;
        
        if ( habilita = '1') then
          estado_siguiente <= captura_datos;
          p_operacion <= calcula_outmsg;	
        else
          estado_siguiente <= inicio;	
        end if;		
        
      when captura_datos =>  -- Estado que inicia una serie de subestados que deshacen las ultimas permutaciones del DES
        
        case operacion is
          
          when calcula_outmsg =>  --Se deshace la ultima permutacion del algoritmo
            
            p_outmsg_gold <=  outdata_gold(57) & outdata_gold(49)  & outdata_gold(41) & outdata_gold(33) & outdata_gold(25) & outdata_gold(17) & outdata_gold(9) & outdata_gold(1) &
                              outdata_gold(59) & outdata_gold(51)  & outdata_gold(43) & outdata_gold(35) & outdata_gold(27) & outdata_gold(19) & outdata_gold(11) & outdata_gold(3) &
                              outdata_gold(61) & outdata_gold(53)  & outdata_gold(45) & outdata_gold(37) & outdata_gold(29) & outdata_gold(21) & outdata_gold(13) & outdata_gold(5) &
                              outdata_gold(63) & outdata_gold(55)  & outdata_gold(47) & outdata_gold(39) & outdata_gold(31) & outdata_gold(23) & outdata_gold(15) & outdata_gold(7) &
                              outdata_gold(56) & outdata_gold(48)  & outdata_gold(40) & outdata_gold(32) & outdata_gold(24) & outdata_gold(16) & outdata_gold(8) & outdata_gold(0) &
                              outdata_gold(58) & outdata_gold(50)  & outdata_gold(42) & outdata_gold(34) & outdata_gold(26) & outdata_gold(18) & outdata_gold(10) & outdata_gold(2) &
                              outdata_gold(60) & outdata_gold(52)  & outdata_gold(44) & outdata_gold(36) & outdata_gold(28) & outdata_gold(20) & outdata_gold(12) & outdata_gold(4) &
                              outdata_gold(62) & outdata_gold(54)  & outdata_gold(46) & outdata_gold(38) & outdata_gold(30) & outdata_gold(22) & outdata_gold(14) & outdata_gold(6);
                  
            p_outmsg_faulty <=  outdata_faulty(57) & outdata_faulty(49)  & outdata_faulty(41) & outdata_faulty(33) & outdata_faulty(25) & outdata_faulty(17) & outdata_faulty(9) & outdata_faulty(1) &
                                outdata_faulty(59) & outdata_faulty(51)  & outdata_faulty(43) & outdata_faulty(35) & outdata_faulty(27) & outdata_faulty(19) & outdata_faulty(11) & outdata_faulty(3) &
                                outdata_faulty(61) & outdata_faulty(53)  & outdata_faulty(45) & outdata_faulty(37) & outdata_faulty(29) & outdata_faulty(21) & outdata_faulty(13) & outdata_faulty(5) &
                                outdata_faulty(63) & outdata_faulty(55)  & outdata_faulty(47) & outdata_faulty(39) & outdata_faulty(31) & outdata_faulty(23) & outdata_faulty(15) & outdata_faulty(7) &
                                outdata_faulty(56) & outdata_faulty(48)  & outdata_faulty(40) & outdata_faulty(32) & outdata_faulty(24) & outdata_faulty(16) & outdata_faulty(8) & outdata_faulty(0) &
                                outdata_faulty(58) & outdata_faulty(50)  & outdata_faulty(42) & outdata_faulty(34) & outdata_faulty(26) & outdata_faulty(18) & outdata_faulty(10) & outdata_faulty(2) &
                                outdata_faulty(60) & outdata_faulty(52)  & outdata_faulty(44) & outdata_faulty(36) & outdata_faulty(28) & outdata_faulty(20) & outdata_faulty(12) & outdata_faulty(4) &
                                outdata_faulty(62) & outdata_faulty(54)  & outdata_faulty(46) & outdata_faulty(38) & outdata_faulty(30) & outdata_faulty(22) & outdata_faulty(14) & outdata_faulty(6);		   
            
            p_operacion <= calcula_diferencia;
            estado_siguiente <= captura_datos;
            
          when  calcula_diferencia => 			
            
            p_diferencia <= outmsg_gold xor outmsg_faulty ;
            p_operacion <= calcula_dif_out_Feistel;
            estado_siguiente <= captura_datos;
            
          when calcula_dif_out_Feistel =>		
            
            p_error <=  diferencia(32 to 63 );
            p_dif_out_Feistel <= diferencia(0 to 31) ;
            p_operacion <= calcula_p_inversa;
            estado_siguiente <= captura_datos;
            
          when calcula_p_inversa =>		--Se calcula la permutacion inversa que sigue a las cajas de sustitucion
            
            p_p_inversa <= dif_out_Feistel(8) 	&  dif_out_Feistel(16) & dif_out_Feistel(22) & dif_out_Feistel(30) & dif_out_Feistel(12) & dif_out_Feistel(27) & dif_out_Feistel(1)  & dif_out_Feistel(17) &
									dif_out_Feistel(23)  & 	dif_out_Feistel(15) & dif_out_Feistel(29) & dif_out_Feistel(5)	 & dif_out_Feistel(25) & dif_out_Feistel(19) & dif_out_Feistel(9)  & dif_out_Feistel(0)  &
									dif_out_Feistel(7) 	&  dif_out_Feistel(13) & dif_out_Feistel(24) & dif_out_Feistel(2)  & dif_out_Feistel(3)  & dif_out_Feistel(28) & dif_out_Feistel(10) & dif_out_Feistel(18) &
									dif_out_Feistel(31)  & 	dif_out_Feistel(11) & dif_out_Feistel(21) & dif_out_Feistel(6)  & dif_out_Feistel(4)  & dif_out_Feistel(26) & dif_out_Feistel(14) & dif_out_Feistel(20);
            
            estado_siguiente <= inicia_key;    
            
          when others =>
            estado_siguiente <= captura_datos;
            p_operacion <= calcula_outmsg;
            
        end case ;	
        
      when inicia_key =>
        
        p_habilita_cont <= '1';
	
        for b in 0 to 7 loop
          p_indice_aux(b) <= 0;
        end loop;
	
        estado_siguiente <= comprueba_key;
        
      when  comprueba_key =>
        
        if ( fin_key_aux = '1') then				--Se comprueba si ha saturado el contador que funciona como generador de subclaves para cada caja 
          estado_siguiente <= almacena_key;
          p_habilita_cont <= '0';
        else
          estado_siguiente <= comprueba_key;		--Se repite el estado de  comprobacin de claves hasta llegar al limite
			 
			--Bucle: Se comprueba si el error ha llegado a todas las cajas de sustitucion y entonces una por una se va comprobando si se produce alguna coincidencia 
          for a in 0 to 7 loop
            if (indice_aux(a) <= 20 ) then 																
              if ((error(indice_error(4*a))   or error(indice_error(4*a+5)) or 
						 error(indice_error(4*a+1)) or error(indice_error(4*a+2)) or 
						 error(indice_error(4*a+3)) or error(indice_error(4*a+4))) = '1') then    --Si algunos de los bits de entrada de alguna caja se activa se empieza a comprobar
                if ( (s_aux1(4*a to 4*a+3) xor s_aux2(4*a to 4*a+3)) = p_inversa (4*a to 4*a+3))  then
                  p_mem(a,indice_aux(a)) <= key_aux  ;   --Se almacena en la tabla "mem" la clave que ha originado la coincidencia
                  p_indice_aux(a) <= indice_aux(a) + 1;	
                end if;				
              end if;		
            end if; 
          end loop;
          
        end if;
        
      when almacena_key =>
        
        remove_array(p_mem);
        
        if (primera_inyeccion = '1') then			 
          p_mem1 <= mem;						--Se copia las coinciencias del actual proceso en la tabla "mem1"
          p_primera_inyeccion <= '0';		
          estado_siguiente <= inicio;		--Se vuelve al estado inicial donde el proceso comienza de nuevo
          p_rdy <= '1';							
        else
          p_mem2 <= mem;							-- A partir del primer ataque las claves coincidentes se almacenan en la tabla "mem2"
          estado_siguiente <= compara_key;
          
          for b in 0 to 7 loop
            p_i(b) <= 0;
            p_j(b) <= 0;
            p_k(b) <= 0;
          end loop;
          
        end if;
        
      when compara_key =>

	if ((rdy_subkey(0) and rdy_subkey(1) and rdy_subkey(2) and 			
		  rdy_subkey(3) and rdy_subkey(4) and rdy_subkey(5) and 
		  rdy_subkey(6) and rdy_subkey(7) )= '1') then
          if (k(0)=1 and k(1)=1 and k(2)=1 and k(3)=1 and k(4)=1 and k(5)=1 and k(6)=1 and k(7)=1) then   --Se comprueba si las claves de cada caja es unica
            if (pruebas_aleatorias = '1') then
				  estado_siguiente <= inicio;
				  p_rdy_subkey_round16 <= '1';
				  p_primera_inyeccion <= '1';
				else
				  estado_siguiente <= calcula_key_entrada;
              p_outkey16 <= mem_coincide(0,0) & mem_coincide(1,0) & mem_coincide(2,0)& 
								    mem_coincide(3,0) & mem_coincide(4,0) & mem_coincide(5,0)& 
								    mem_coincide(6,0) & mem_coincide(7,0);	
				end if;	  
            remove_array(p_mem);
            remove_array(p_mem1);
            remove_array(p_mem2);
            remove_array(p_mem_coincide);
          else
            p_mem1 <= mem_coincide;
            estado_siguiente <= inicio;
            p_rdy <= '1';
            remove_array(p_mem2);
            remove_array(p_mem_coincide);
          end if;
          
	else
          estado_siguiente <= compara_key;
       --Bucle: Se encarga de comprobar las coincidencias entre las tablas: "mem1" y "mem2"         
          for a in 0 to 7 loop
            
            if(rdy_subkey(a) = '0') then
              if(mem1(a,0) = "UUUUUU") then
                copycolumn(orig=>mem2,col=>a,dest=>p_mem_coincide);
                p_rdy_subkey(a) <= '1';
              elsif(mem2(a,0) ="UUUUUU") then
                copycolumn(orig=>mem1,col=>a,dest=>p_mem_coincide);
                p_rdy_subkey(a) <= '1';            
              else
                if (i(a)>=0 and i(a)<=20) then
                  if (mem1(a,i(a)) /= "UUUUUU") then
                    if (mem1(a,i(a))>=mem2(a,j(a)))then
                      if(j(a)>=0 and j(a)<=20) then
                        if(mem2(a,j(a)) /= "UUUUUU") then	  
                          if(mem1(a,i(a)) = mem2(a,j(a))) then
                            p_mem_coincide(a,k(a)) <= mem1(a,i(a));
                            p_k(a) <= k(a)+1;
                            p_i(a) <= i(a)+1;
                            p_j(a) <= 0;
                          else
                            p_j(a)<= j(a)+1;
                          end if;
                        else
                          p_i(a)<= i(a)+1;
                          p_j(a) <= 0;
                        end if;
                      else
                        p_i(a)<= i(a)+1;
                        p_j(a)<= 0;
                      end if;
                    else
                      p_i(a) <= i(a)+1;
                      p_j(a) <= 0;
                    end if;
                  else
                    p_i(a) <= 0;
                    p_j(a) <= 0;
                    p_rdy_subkey(a) <= '1';
                  end if;
                else
                  p_i(a) <= 0;
                  p_j(a) <= 0;
                  p_rdy_subkey(a) <= '1';		
                end if;
              end if;
            end if;
				
          end loop;	
	end if;
        
      when calcula_key_entrada =>		--Este estado deshace las permutaciones de la clave de la ronda 16 hasta el principio
        
	mykey <= outkey16(0)  & outkey16(2  to 5)  & outkey16(1)  &
            outkey16(6)  & outkey16(8  to 11) & outkey16(7)  &
            outkey16(12) & outkey16(14 to 17) & outkey16(13) &
            outkey16(18) & outkey16(20 to 23) & outkey16(19) &
            outkey16(24) & outkey16(26 to 29) & outkey16(25) &
            outkey16(30) & outkey16(32 to 35) & outkey16(31) &
            outkey16(36) & outkey16(38 to 41) & outkey16(37) &
            outkey16(42) & outkey16(44 to 47) & outkey16(43);
        
	keylr <= mykey(4) & mykey(23) & mykey(6) & mykey(15) & mykey (5) & mykey(9) & mykey (19) & mykey (17) &
                 'U' & mykey(11) & mykey(2) & mykey(14) & mykey(22) & mykey(0) & mykey(8) & mykey(18) & mykey(1) & 
                 'U' & mykey(13) & mykey(21) & mykey(10) & 'U'  & mykey(12) & mykey(3) & 'U' & mykey (16)  & mykey(20) & mykey(7)  &
                 mykey(46) & mykey(30) & mykey(26) & mykey(47) & mykey(34) & mykey(40) & 'U'& mykey(45) & mykey(27)  & 'U' &
                 mykey(38) & mykey(31)  & mykey(24)  & mykey(43) & 'U' & mykey(36) & mykey(33)  & mykey(42)  & mykey(28)  & 
                 mykey(35)  & mykey(37)  & mykey(44)   & mykey(32)   & mykey(25)  & mykey(41) & 'U'  & mykey(29)& mykey(39);				
        
    key_l (0 to 27) <= keylr(1 to 27) & keylr (0);
    key_r (0 to 27) <= keylr(29 to 55) & keylr ( 28);
        
    outkey <=  key_l(6)  & key_l(14)  & key_l(22)   & key_r(26) & key_r(22) & key_r(14) & key_r(6)  & 
               'U' 		 & key_l(5)   & key_l(13)   & key_l(21) & key_r(25) & key_r(21) & key_r(13) & 
					key_r(5)  &    'U' 	  & key_l(4)    & key_l(12) & key_l(20) & key_r(24) & key_r(20) & 
					key_r(12) & key_r(4)   &    'U'      & key_l(3)  & key_l(11) & key_l(19) & key_r(23) & 
					key_r(19) & key_r(11)  & key_r(3)    &   'U' 	 & key_l(2)  & key_l(10) & key_l(18) & 
					key_l(26) & key_r(18)  & key_r(10)   & key_r(2)  &    'U' 	 & key_l(1)  & key_l(9)  & 
					key_l(17) & key_l(25)  & key_r(17)   & key_r(9)  & key_r(1)  &    'U'    & key_l(0)  & 
					key_l(8)  & key_l(16)  & key_l(24)   & key_r(16) & key_r(8)  & key_r(0)  &   'U'     & 
					key_l(27) & key_l(7)   & key_l(15)   & key_l(23) & key_r(15) & key_r(7)  & key_r(27) & 'U'; 		
        
        p_rdy_subkey_round16 <= '1'; 
        estado_siguiente <= fin;
        
      when fin =>
        
        estado_siguiente <= fin;
        p_rdy_subkey_round16 <= '0';
        
      when others =>
		
        estado_siguiente <= inicio;
        
    end case;

  end process;
  
  sinc:process(rst ,clk)
  begin
    
    if (rst = '1') then
      
      estado_actual <= inicio;
      operacion <= calcula_outmsg;
      habilita_cont <= '0';
      
      primera_inyeccion <= '1';
      
      for a in 0 to 7 loop
        i(a) <= 0;
        j(a) <= 0;
        k(a) <= 0;
        rdy_subkey(a) <= '0';			
      end loop;
      
      for c in 0 to 31 loop
        indice_error(c+1) <= c;
      end loop;
      indice_error(0) <= 31;
      indice_error(33) <= 0;
      
      rdy <= '0';
      rdy_subkey_round16 <= '0';
      
    elsif(clk = '1' and clk'event)then
      
      habilita_cont <= p_habilita_cont;
      indice_aux <= p_indice_aux;
      primera_inyeccion <= p_primera_inyeccion;
      mem <= p_mem;
      outmsg_gold <= p_outmsg_gold;
      outmsg_faulty <= p_outmsg_faulty;
      diferencia <= p_diferencia;
      error <= p_error;
      dif_out_Feistel <= p_dif_out_Feistel;
      p_inversa <= p_p_inversa;
      outkey16 <= p_outkey16;
      estado_actual <= estado_siguiente;
      operacion <= p_operacion;
      mem1 <= p_mem1;
      mem2 <= p_mem2;
      mem_coincide <= p_mem_coincide;
      k <= p_k;
      i <= p_i;
      j <= p_j;
      rdy <= p_rdy;
      rdy_subkey <= p_rdy_subkey;
      rdy_subkey_round16 <= p_rdy_subkey_round16;
      
    end if;
      
  end process; 
  
  
end Behavioral;

